#!/usr/bin/python
# -*- coding: utf-8 -*-

import codecs
import re
import gzip
from datetime import datetime,timedelta
import sys
import urllib

# 0:ユニークID

# 1:アクセス先ドメイン名（※）
# 2:アクセス元のホスト名もしくはIPアドレス
# 3:クライアント側のユーザー名（通常は空欄）
# 4:認証を行った際のユーザー名（通常は空欄）
# 5:アクセス日時
# 6:クライアントがアクセスした際のリクエスト内容
# 7:リクエストに対するステータスコード
# 8:実際に転送したデータ量
# 9:リファラ（どのリンクから辿ってきたのか）
#10:ユーザエージェント

class AccessLog:
	def __init__(self, filename):

		self.rawLogs = []

		if filename.endswith('gz') == True:
			f = gzip.open(filename, 'r')
		else:
			f = open(filename, 'r')

		contents = f.read()
		lines = contents.splitlines()

		re_line = re.compile('^(.*?)\s(.*?)\s(.*?)\s(.*?)\s\[(.*?)\]\s"(.*?)"\s(.*?)\s(.*?)\s"(.*?)"\s"(.*?)"')

		id = 0

		for line in lines:
			match = re_line.match(line)
			if match != None:
				t = [id, match.group(1), match.group(2), match.group(3), match.group(4), match.group(5), match.group(6), match.group(7), match.group(8), match.group(9), match.group(10)]
				self.rawLogs.append(t)
				id = id + 1
		f.close()


# 0: ユニークID
# 1: ドメイン名
# 2: ホスト名もしくはIPアドレス
# 3: アクセス日時
# 4: リクエスト
# 5: リファラ
# 6: 検索エンジン
# 7: OS
# 8: ブラウザ

		self.analyzeLogs = []

		for log in self.rawLogs:
			
			request = self.analyzeRequest(log[6])
			time = self.analyzeTime(log[5])
			refer = self.analyzeRefer(log[9])
			search = self.analyzeSearch(log[9])												
			os = self.analyzeOS(log[10])
			userAgent = self.analyzeBrowser(log[10])

			t = [log[0], log[1], log[2], time, request, refer, search, os, userAgent]
			self.analyzeLogs.append(t)


		self.analyzeHostData()
		self.analyzeTimeData()
		self.analyzeRequestData()
		self.analyzeReferData()
		self.analyzeSearchData()
		self.analyzeOSData()
		self.analyzeBrowserData()

#
# 'GET /osx/img/back_title.png HTTP/1.1'
#
# [True, False, 'GET', '/osx/img/back_title.png']
#
# 'GET /osx/ HTTP/1.1'
#
# [True, True, 'GET', '/osx/']
#
	def analyzeRequest(self, request):
		re_request = re.compile('^(.*?)\s(.*?)\s(.*?)')
	
		ret = [False, '']

		match = re_request.match(request)

		if match != None:
			op = match.group(1)
			path = match.group(2)
		else:
			op = ''
			path = request
			return [False, False, op, path]		
		
		if path.find('/cgi-bin/', 0, 9) != -1:
			return [True, False, op, path]
		elif path.lower().endswith('htm') == False and path.lower().endswith('html') == False and path.lower().endswith('/') == False and path.lower().endswith('zip') == False:
			return [True, False, op, path]
		
		return [True, True, op, path]

#
#
#
# str = '02/Dec/2010:22:14:38 +0900'
#
	def analyzeTime(self, str):

#		offset = str[-5:]
#		delta = timedelta(hours = int(offset) / 100)

		time = datetime.strptime(str[:-6],'%d/%b/%Y:%H:%M:%S')
#		time -= delta

		t = time.timetuple()
		m = [t[0], t[1], t[2], t[3], t[4], t[5]]
#		print time.isoformat(' ')
	
		return m
#
# 5: リファラ
#
#  [True, True, '-']
#
	def analyzeRefer(self, refer):

		if refer.find('.sakura.ne.jp') != -1:
			return [False, False, refer]

		if refer == '-':
			return [True, False, refer]

		return [True, True, refer]

#
#
#
# [True, Google, keywords, refer]
#
	def analyzeSearch(self, refer):

		if refer.find('sakura.ne.jp') != -1 or refer == '-' or refer.endswith('html') or refer.endswith('/'):
			return [False, '', '', refer]

		if refer.find('google.') != -1:
			re_search = re.compile('^.*?[\?&]q=(.*?)&')

			match = re_search.match(refer)

			if match != None:
				r = match.group(1)
				r = r.replace('+', ' ')
			else:
				re_search = re.compile('^.*?[\?&]q=(.*?)$')

				match = re_search.match(refer)

				if match != None:
					r = match.group(1)			
					r = r.replace('+', ' ')
				else:
					r = refer

			r = urllib.unquote(r)

			return [True, 'Google', r, refer]

		elif refer.find('yahoo.') != -1:
			re_search = re.compile('^.*?[\?&]p=(.*?)&')

			match = re_search.match(refer)

			if match != None:
				r = match.group(1)
				r = r.replace('+', ' ')				
			else:
				re_search = re.compile('^.*?[\?&]p=(.*?)$')

				match = re_search.match(refer)

				if match != None:
					r = match.group(1)
					r = r.replace('+', ' ')
				else:
					r = refer

			r = urllib.unquote(r)

			return [True, 'Yahoo', r, refer]

		else:
			r = urllib.unquote(refer)

			return [True, 'Other', r, refer]
#
#
#
#
	def analyzeOS(self, os):
#
#	USER_AGENT	:	Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5
#
		if os.find('Macintosh') != -1:
			re_os = re.compile('.*Macintosh;\sU;\s(.*?\sMac\sOS\sX\s[\d|_|\.]*);')
			match = re_os.match(os)
			if match != None:
				r = match.group(1)
			else:
				r = os

			return [True, 'Macintosh', r]
				
#
#	USER_AGENT	:	Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; ja-jp) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10
#
		elif os.find('iPad') != -1:
			re_os = re.compile('.*iPad;\sU;\s.*?\s(OS\s[\d|_|\.]*)\slike')
			match = re_os.match(os)
			if match != None:
				r = match.group(1)
			else:
				r = os
					
			return [True, 'iPad', r]

#
#	USER_AGENT	:	Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; ja-jp) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117
#
		elif os.find('iPod') != -1:
			re_os = re.compile('.*iPod;\sU;\s.*?\s(.*?\sOS\s[\d|_|\.]*)\slike')
			match = re_os.match(os)
			if match != None:
				r = match.group(1)
			else:
				r = os
		
			return [True, 'iPod', r]
#
#	USER_AGENT	:	Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Mobile/7E18
#
		elif os.find('iPhone') != -1:
			re_os = re.compile('.*iPhone;\sU;\s.*?\s(.*?\sOS\s[\d|_|\.]*)\slike')
			match = re_os.match(os)
			if match != None:
				r = match.group(1)
			else:
				r = os
			return [True, 'iPhone', r]
#
#	USER_AGENT	:	Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)
#
		elif os.find('Windows') != -1:
			re_os = re.compile('.*;\s(Windows\sNT\s[\d|\.]+);')
			match = re_os.match(os)
			if match != None:
				r = match.group(1)
			else:
				re_os = re.compile('.*;\s(Windows\sNT\s[\d|\.]+).*$')
				match = re_os.match(os)
				if match != None:
					r = match.group(1)
				else:
					r = os
		
			if r == 'Windows NT 5.1':
				r = 'Windows XP'
			elif r == 'Windows NT 5.2':
				r = 'Windows XP (64bit)'
			elif r == 'Windows NT 6.0':
				r = 'Windows Vista'
			elif r == 'Windows NT 6.1':
				r = 'Windows 7'
			elif r == 'Windows NT 5.0':
				r = 'Windows 2000'
			
			return [True, 'Windows', r]
#
#
#
		else:
			return [True, 'Other', os]

#
# "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; ja-JP-mac; rv:1.9.2.12) Gecko/20101026 Firefox/3.6.12"
# "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; ja-jp) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4"
# "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C)"
#
	def analyzeBrowser(self, userAgent):
		
		re_userAgent = re.compile('^.*Safari/([\d\.]+)')
	
		match = re_userAgent.match(userAgent)

		if match != None:
			version = match.group(1)
			return ['Safari', version]

		re_userAgent = re.compile('^.*Firefox/([\d\.]+)')
	
		match = re_userAgent.match(userAgent)

		if match != None:
			version = match.group(1)
			return ['Firefox', version]

		re_userAgent = re.compile('^.*MSIE\s+([\d\.]+)')
	
		match = re_userAgent.match(userAgent)

		if match != None:
			version = match.group(1)
			return ['MSIE', version]

		return ['Other', userAgent]

# 0: ユニークID
# 1: ドメイン名
# 2: ホスト名もしくはIPアドレス
# 3: アクセス日時
# 4: リクエスト
# 5: リファラ
# 6: 検索エンジン
# 7: OS
# 8: ブラウザ

#
#	[[number, hostname, [id, id, ...]] ...]
#
	def matchHostId(self, data, hostname, line):
	
		for t in data:
			if t[1] == hostname:
				t[0] = t[0] + 1
				t[2].append(line[0])
				return
		data.append([1, hostname, [line[0]]])

#
#
# [ [kyoshiaki.sakura.ne.jp, [matchHostId...] ] ...]  
#
	def analyzeHostData(self):	

		self.hostData = []
				
		for line in self.analyzeLogs:

			flag = False
			for i in self.hostData: 
				if i[0] == line[1]:
					self.matchHostId(i[1], line[2], line)
					flag = True
					break

			if flag == False:
				self.hostData.append([ line[1], [[1, line[2], [line[0]]]]])

		for data in self.hostData:
			data[1] = sorted(data[1], lambda x, y: cmp(x[0], y[0]), None, True)
#
#
#
	def printHostData(self, flag = False):

		for i in self.hostData:
			length = len(i[0]) + len(str(len(i[1]))) + 2
			width = (76 - length) / 2
			
			print '%s %s (%d) %s' % ('-' *  width, i[0], len(i[1]), '-' *  width)

			for j in i[1]:
				if j[1].lower().endswith('.org') or j[1].lower().endswith('.edu') or j[1].lower().endswith('.gov') or j[1].lower().endswith('.mil'):
					print '%s%5d   %s%s' % (self.printESC('red'), j[0], j[1], self.printESC('revert'))
				elif j[1].lower().endswith('.ac.jp') or j[1].lower().endswith('.go.jp'):
					print '%s%5d   %s%s' % (self.printESC('blue'), j[0], j[1], self.printESC('revert'))
				else:
					print '%s%5d   %s%s' % (self.printESC('revert'), j[0], j[1], self.printESC('revert'))
			
				if flag == True:
					r = ''
					for k in j[2]:
						r += ' ' + str(k)
					print '%s %s' % (' ' * 16, r)

#
#
# [ [kyoshiaki.sakura.ne.jp,[[0, [id...] ],[1, [id...]],...,23]], ...]
#
	def analyzeTimeData(self):

		self.timeData = []

		for line in self.analyzeLogs:
			time = line[3]
			flag = False

			for i in range(len(self.timeData)):
				m = self.timeData[i]
				
				if m[0] == line[1]:
					count = m[1]
					count[time[3]][0] += 1
					count[time[3]][1].append(line[0])
					flag = True
					break
			if flag == False:

				hour = []
	
				for j in range(24):
					item = []
	
					if j == time[3]:
						item.append(line[0])
						count = 1
					else:
						count = 0
						
					hour.append([count, item])
			
				self.timeData.append([line[1], hour])

#	
#
#
	def printTimeData(self, flag=False):	
		for i in self.timeData:
			length = len(i[0])
			width = (76 - length) / 2
			
			print '%s %s %s' % ('-' *  width, i[0], '-' *  width)
			for j in range(24):
				item = i[1]
				d = item[j]
				print "%s %2d: %3d %s" % (' ' * 3, j, d[0], "*" * d[0])

				if flag:
					for k in d[1]:
						print '%d ' % k,
				if flag:
					print ' '
					

#
#
# [ [kyoshiaki.sakura.ne.jp, [[1, request, matchHostId] ...] ] ...]  

	def analyzeRequestData(self, allFlag = False):

		self.requestData = []

		for line in self.analyzeLogs:

			request = line[4]

			if request[0] == False:
				continue
			if request[1] == False and allFlag == False:
				continue

			eflag = False
			for i in self.requestData: 
				if i[0] == line[1]:
					eflag = True
					flag = False
					for j in range(len(i[1])):
						m = i[1][j]
						if m[1] == request[3]:
							m[0] += 1
							self.matchHostId(m[2], line[2], line)
							flag = True
							break
					if flag == False:
						i[1].append([1, request[3], [[1, line[2], [line[0]] ] ]])
			if eflag == False:
				self.requestData.append([ line[1], [[1, request[3], [[1, line[2], [line[0]]]]]]])


		for i in self.requestData:
			for j in i[1]:
				j[2] = sorted(j[2], lambda x, y: cmp(x[0], y[0]), None, True)
			i[1] = sorted(i[1], lambda x, y: cmp(x[0], y[0]), None, True)
#
#
#

	def printRequestData(self, flag = False):
		for i in self.requestData:
			count = 0
			access = []
			for j in i[1]:
				count += len(j[2])
				
				for k in j[2]:
					if not k[1] in access:
						access.append(k[1])

			length = len(i[0]) + len(str(count)) + len(str(len(access)))
			width = (73 - length) / 2			
			print '%s %s (%d:%d) %s' % ('-' *  width, i[0], count, len(access), '-' *  width)

			for j in i[1]: 
			
				if j[1].lower().endswith('zip'):
					print '%s%s   %s%s' % (self.printESC('red'), j[0], j[1], self.printESC('revert'))
				else:
					print '%s%s   %s%s' % (self.printESC('blue'), j[0], j[1], self.printESC('revert'))

				for k in j[2]:
				
					print '%s %5d   %s' % (' ' * 5, k[0], k[1])
					
					if flag == True:
						r = ''
						for l in k[2]:
							r += ' ' + str(l)
						print '%s %s' % (' ' * 12, r)

#
#
# [ [kyoshiaki.sakura.ne.jp, [[1, refer, matchHostId] ...] ] ...]  
#
	def analyzeReferData(self):
		self.referData = []

		for line in self.analyzeLogs:

			refer = line[5]

			if refer[0] == False:
				continue

			eflag = False
			for i in self.referData: 
				if i[0] == line[1]:
					eflag = True
					flag = False
					for j in range(len(i[1])):
						m = i[1][j]
						if m[1] == refer[2]:
							m[0] += 1
							self.matchHostId(m[2], line[2], line)
							flag = True
							break
					if flag == False:
						i[1].append([1, refer[2], [[1, line[2], [line[0]] ] ]])
			if eflag == False:
				self.referData.append([ line[1], [[1, refer[2], [[1, line[2], [line[0]]]]]]])

		for i in self.referData:
			for j in i[1]:
				j[2] = sorted(j[2], lambda x, y: cmp(x[0], y[0]), None, True)
			i[1] = sorted(i[1], lambda x, y: cmp(x[0], y[0]), None, True)
#
#
#
	def printReferData(self, flag = False):
		for i in self.referData:
			length = len(i[0])
			width = (76 - length) / 2
			
			print '%s %s %s' % ('-' *  width, i[0], '-' *  width)
			for j in i[1]: 
				re_refer = re.compile('^(http://.*?/)(.*)')
				match = re_refer.match(j[1])

				if match != None:
					print '%5d   %s%s%s%s%s' % (j[0], self.printESC('lightblue'), match.group(1), self.printESC('revert'), match.group(2), self.printESC('revert'))
				else:
					print '%5d   %s' % (j[0], j[1])

#				for k in j[2]:
#					print '%s %5d   %s' % (' ' * 5, k[0], k[1])
#					
#					if flag == True:
#						r = ''
#						for p in k[2]:
#							r += ' ' + str(p)
#						print '%s %s' % (' ' * 16, r)

# Google, Yahoo, Other
#							 Google								    Yahoo								Other
# [ [kyoshiaki.sakura.ne.jp, [ [[1, 'PC-G850V', matchHostId] ...], [[1, ,'keyword',  matchHostId] ...],[[1, 'keyword', matchHostId] ...] ] ...]
#
	def analyzeSearchData(self):
		self.searchData = []

		for line in self.analyzeLogs:

			search = line[6]

			if search[0] == False:
				continue


			eflag = False
			for i in self.searchData: 
				if i[0] == line[1]:
					eflag = True
					m = i[1][2]
					if search[1] == 'Google':
						m = i[1][0] 
					elif search[1] == 'Yahoo':
						m = i[1][1] 
					else:
						m = i[1][2] 
					
					flag = False
					for j in range(len(m)):
						n = m[j]
						if n[1] == search[2]:
							flag = True
							n[0] += 1
							self.matchHostId(n[2], line[2], line)
							break
					if flag == False:
						m.append([1, search[2], [[1, line[2], [line[0]] ]]])

			if eflag == False and search[1] == 'Google':
				self.searchData.append([ line[1], [ [[1, search[2], [[1, line[2], [line[0]]]]]], [], [] ] ])
			elif eflag == False and search[1] == 'Yahoo':
				self.searchData.append([ line[1], [ [], [[1, search[2], [[1, line[2], [line[0]]]] ]], [] ]])
			elif eflag == False:

				self.searchData.append([ line[1], [ [], [], [[1, search[2], [[1, line[2], [line[0]]]]]] ] ])

		for i in self.searchData:
			for j in i[1]:
				for k in j:
					k[2] = sorted(k[2], lambda x, y: cmp(x[0], y[0]), None, True)
				j = sorted(j, lambda x, y: cmp(x[0], y[0]), None, True)
#
#
#
	def printSearchData(self, flag = False):
		for i in self.searchData:
			length = len(i[0])
			width = (76 - length) / 2
			
			print '%s %s %s' % ('-' *  width, i[0], '-' *  width)
			s = i[1]
			
			for j in range(len(s)):
				l = s[j]
				space = ' ' * 5
				if j == 0:
					print '%s%s Google%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 1:
					print '%s%s Yahoo%s' % (self.printESC('green'), space, self.printESC('revert'))
				else:
					print '%s%s Other%s' % (self.printESC('green'), space, self.printESC('revert'))

				for k in l:
					print '%s%s %5d   "%s%s"' % (self.printESC('blue'), ' ' * 5, k[0], k[1], self.printESC('revert'))
					for o in k[2]:
						print '%s %5d   %s' % (' ' * 10, o[0], o[1])

						if flag == True:
							r = ''
							for p in o[2]:
								r += ' ' + str(p)
							print '%s %s' % (' ' * 17, r)

#						
#							 Macintosh,						            iPad,								iPod, iPhone, Windows, Other						
# [ [kyoshiaki.sakura.ne.jp, [ [[1, 'Mac OS 10_6_5', matchHostId] ...], [[1, ,'keyword',  matchHostId] ...],[[1, 'keyword', matchHostId] ...], ] ...]
#
	def analyzeOSData(self):
		self.osData = []

		for line in self.analyzeLogs:

			os = line[7]

			if os[0] == False:
				continue


			eflag = False
			for i in self.osData: 
				if i[0] == line[1]:
					eflag = True
					m = i[1][5]
					if os[1] == 'Macintosh':
						m = i[1][0] 
					elif os[1] == 'iPad':
						m = i[1][1] 
					elif os[1] == 'iPod':
						m = i[1][2] 
					elif os[1] == 'iPhone':
						m = i[1][3] 
					elif os[1] == 'Windows':
						m = i[1][4] 
					else:
						m = i[1][5] 
					
					flag = False
					for j in range(len(m)):
						n = m[j]
						if n[1] == os[2]:
							flag = True
							n[0] += 1
							self.matchHostId(n[2], line[2], line)
							break
					if flag == False:
						m.append([1, os[2], [[1, line[2], [line[0]] ]]])

			if eflag == False and os[1] == 'Macintosh':
				self.osData.append([ line[1], [ [[1, os[2], [[1, line[2], [line[0]]]]]], [], [], [], [], [] ] ])
			elif eflag == False and os[1] == 'iPad':
				self.osData.append([ line[1], [ [], [[1, os[2], [[1, line[2], [line[0]]]] ]], [], [], [], [] ]])
			elif eflag == False and os[1] == 'iPod':
				self.osData.append([ line[1], [ [], [], [[1, os[2], [[1, line[2], [line[0]]]] ]], [], [], [] ]])
			elif eflag == False and os[1] == 'iPhone':
				self.osData.append([ line[1], [ [], [], [], [[1, os[2], [[1, line[2], [line[0]]]] ]], [], [] ]])
			elif eflag == False and os[1] == 'Windows':
				self.osData.append([ line[1], [ [], [], [], [], [[1, os[2], [[1, line[2], [line[0]]]] ]], [] ]])
			elif eflag == False:
				self.osData.append([ line[1], [ [], [], [], [], [], [[1, os[2], [[1, line[2], [line[0]]]]]] ] ])

		for i in self.osData:
			for j in range(len(i[1])):
				for k in i[1][j]:
					k[2] = sorted(k[2], lambda x, y: cmp(x[0], y[0]), None, True)
				i[1][j] = sorted(i[1][j], lambda x, y: cmp(x[0], y[0]), None, True)
#
#
#
#
#
#
#
	def printOSData(self):
		for i in self.osData:
			length = len(i[0])
			width = (76 - length) / 2
			
			print '%s %s %s' % ('-' *  width, i[0], '-' *  width)
			s = i[1]
			
			for j in range(len(s)):
				l = s[j]
				space = ' ' * 3
				if j == 0:
					print '%s%s Macintosh%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 1:
					print '%s%s iPad%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 2:
					print '%s%s iPod%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 3:
					print '%s%s iPhone%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 4:
					print '%s%s Windows%s' % (self.printESC('green'), space, self.printESC('revert'))
				else:
					print '%s%s Other%s' % (self.printESC('green'), space, self.printESC('revert'))

				for k in l:
					print '%s %5d   %s' % (' ' * 3, k[0], k[1])
#					for o in k[2]:
#						print '%s %d %s' % (' ' * 15, o[0], o[1])


# 0: ユニークID
# 1: ドメイン名
# 2: ホスト名もしくはIPアドレス
# 3: アクセス日時
# 4: リクエスト
# 5: リファラ
# 6: 検索エンジン
# 7: OS
# 8: ブラウザ

#								 Safari,							Firefox,							MSIE,							Other
# [ [kyoshiaki.sakura.ne.jp, [ [[1, '6533.18.5', matchHostId] ...], [[1, ,'3.6.12',  matchHostId] ...],[[1, '8.0', matchHostId] ...], ] ...]
#
#
	def analyzeBrowserData(self):
		self.browserData = []

		for line in self.analyzeLogs:

			browser = line[8]

			eflag = False
			for i in self.browserData: 
				if i[0] == line[1]:
					eflag = True
					m = i[1][3]
					if browser[0] == 'Safari':
						m = i[1][0] 
					elif browser[0] == 'Firefox':
						m = i[1][1] 
					elif browser[0] == 'MSIE':
						m = i[1][2] 
					else:
						m = i[1][3] 
					
					flag = False
					for j in range(len(m)):
						n = m[j]
						if n[1] == browser[1]:
							flag = True
							n[0] += 1
							self.matchHostId(n[2], line[2], line)
							break
					if flag == False:
						m.append([1, browser[1], [[1, line[2], [line[0]] ]]])

			if eflag == False and browser[0] == 'Safari':
				self.browserData.append([ line[1], [ [[1, browser[1], [[1, line[2], [line[0]]]]]], [], [], [] ]])
			elif eflag == False and browser[0] == 'Firefox':
				self.browserData.append([ line[1], [ [], [[1, browser[1], [[1, line[2], [line[0]]]]]], [], [] ]])
			elif eflag == False and browser[0] == 'MSIE':
				self.browserData.append([ line[1], [ [], [], [[1, browser[1], [[1, line[2], [line[0]]]]]], [] ]])
			elif eflag == False:
				self.browserData.append([ line[1], [ [], [], [], [[1, browser[1], [[1, line[2], [line[0]]]]]] ]])

		for i in self.browserData:
			for j in range(len(i[1])):
				for k in i[1][j]:
					k[2] = sorted(k[2], lambda x, y: cmp(x[0], y[0]), None, True)
				i[1][j] = sorted(i[1][j], lambda x, y: cmp(x[0], y[0]), None, True)

#
#
#
	def printBrowserData(self):
		for i in self.browserData:
			length = len(i[0])
			width = (76 - length) / 2
			
			print '%s %s %s' % ('-' *  width, i[0], '-' *  width)
			s = i[1]
			
			for j in range(len(s)):
				l = s[j]
				space = ' ' * 3
				if j == 0:
					print '%s%s Safari%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 1:
					print '%s%s Firefox%s' % (self.printESC('green'), space, self.printESC('revert'))
				elif j == 2:
					print '%s%s MSIE%s' % (self.printESC('green'), space, self.printESC('revert'))
				else:
					print '%s%s Other%s' % (self.printESC('green'), space, self.printESC('revert'))

				for k in l:
					print '%s %5d   %s' % (' ' * 5, k[0], k[1])
#					for o in k[2]:
#						print '%s %d %s' % (' ' * 15, o[0], o[1])


#
#
#
	def printESC(self, color='black'):
		
		if color == 'black':
			return '\x1b[30m';
		elif color == 'red':
			return '\x1b[31m';
		elif color == 'green':
			return '\x1b[32m';
		elif color == 'yellow':
			return '\x1b[33m';
		elif color == 'blue':
			return '\x1b[34m';
		elif color == 'purple':
			return '\x1b[35m';
		elif color == 'lightblue':
			return '\x1b[36m';
		elif color == 'white':
			return '\x1b[37m';
		elif color == 'revert':	
			if sys.platform.find('darwin') != -1:
				return '\x1b[30m';
			elif sys.platform.find('linux') != -1:
				return '\x1b[37m';
			else:
				return '\x1b[30m';
		else:
			if sys.platform.find('darwin') != -1:
				return '\x1b[30m';
			elif sys.platform.find('linux') != -1:
				return '\x1b[37m';
			else:
				return '\x1b[30m';

